<?php
// $Id: pageroute_subform.inc,v 1.1 2007/03/13 13:36:14 fago Exp $

/**
 * @file
 * Provides a new form element, that eases reusing already existing forms
 */

/*
 * Implementation of hook_elements()
 */
function pageroute_elements() {
  $type['subform'] = array(
    '#id' => 'pageroute_subform',
    '#after_build' => array('pageroute_subform_after_build'), 
    '#redirect' => false,
    '#arguments' => array(),
  );
  return $type;
}

/*
 * This function invokes drupal_get_form() once again for the subform.
 */
function pageroute_subform_after_build($form_element, &$form_values) {

  $subform = call_user_func_array('drupal_retrieve_form', array_merge(array($form_element['#id']), $form_element['#arguments']));
  $subform['#token'] = FALSE;

  //if the form has been submitted, set the id to the subform's id
  if (isset($form_element['#post']) && isset($form_element['#post']['form_id'])) {
    $subform['#post'] = $form_element['#post'];
    $subform['#post']['form_id'] = $form_element['#id'];
  }
  pageroute_subform_call('drupal_prepare_form', $form_element['#id'], $subform);
  
  unset($subform['form_id']);
  
  //don't validate the subform in the first place, pageroute_subform_validate() will take care of this
  $subform['#validated'] = TRUE;
  $form_element['#form'] = $subform;
  $form_element['#validate'] = array('pageroute_subform_validate' => array());
  
  //don't render it, again we will take care of this
  $form_element['#type'] = 'subform_element';
  $form_element['#children'] = '';
  $form_element['#form']['#type'] = 'subform'; //set the type back again to subform
  return $form_element;
}

/*
 * Validates the subform element by validating the form
 */
function pageroute_subform_validate($element) {
  global $subforms;
  
  $element['#form']['#validated'] = FALSE;
  pageroute_subform_call('drupal_validate_form', $element['#id'], $element['#form']);
  
  //save subforms in the global, so that the submit handler is able to submit them
  if (!isset($subforms)) {
    $subforms = array();
  }
  $subforms[] = $element['#form'];
}


/*
 * Submits the subforms of a form
 */
function pageroute_subform_submit() {
  global $subforms;
  
  if (isset($subforms)) {
    foreach ($subforms as $subform) {
      if (!isset($subform['#submitted'])) {
        pageroute_subform_call('drupal_submit_form', $subform['#post']['form_id'], $subform);
        $subform['#submitted'] = TRUE;
      }
    }
  }
}

/*
 * theme function for a subform element
 * invoke drupal_render_form like for a normal form
 */
function theme_subform_element(&$element) { 
  return drupal_render_form($$element['id'], $element['#form']);
}

/*
 * Theme function for a subform - Puts a div around its content
 */
function theme_subform(&$element) {
  _form_set_class($element, array('subform'));
  return '<div id="'. $element['#id'] .'">'. $element['#children'] .'</div>';
}

/*
 * Invokes the given function on the given (sub)form
 * This function cares about the globals used by the form system,
 * so that there are always the right globals in place!
 * It's intended to be used with drupal_prepare_form, drupal_validate_form, drupal_submit_form
 */
function pageroute_subform_call($function, $form_id, &$form) {
  global $form_values, $form_submitted, $user, $form_button_counter;
  static $saved_globals = array();

  if (!isset($saved_globals[$form_id])) {
    $saved_globals[$form_id] = array();
    $saved_globals[$form_id]['before'] = array();
    $saved_globals[$form_id]['form_values'] = array();
    $saved_globals[$form_id]['form_submitted'] = FALSE;
    $saved_globals[$form_id]['form_button_counter'] = array(0, 0);
  }

  array_push($saved_globals[$form_id]['before'], array($form_values, $form_submitted, $form_button_counter));

  //restore global variables
  $form_values = $saved_globals[$form_id]['form_values'];
  $form_submitted = $saved_globals[$form_id]['form_submitted'];
  $form_button_counter = $saved_globals[$form_id]['form_button_counter'];

  $function($form_id, $form);
  
  //save global variables for the next time we have to operate on this form_id
  $saved_globals[$form_id]['form_values'] = $form_values;
  $saved_globals[$form_id]['form_submitted'] = $form_submitted;
  $saved_globals[$form_id]['form_button_counter'] = $form_button_counter;

  // We've finished calling functions that alter the global values, so we can
  // restore the ones that were there before this function was called.
  list($form_values, $form_submitted, $form_button_counter) = array_pop($saved_globals[$form_id]['before']);
  
  //propagate if the subform has been submitted
  $form_submitted = $form_submitted || $saved_globals[$form_id]['form_submitted'];
  //also count the form buttons of subforms to the outer form
  if ($function == 'drupal_prepare_form' && $saved_globals[$form_id]['form_button_counter'][0]) {
    $form_button_counter[0] += $saved_globals[$form_id]['form_button_counter'][0];
    $form_button_counter[1] += $saved_globals[$form_id]['form_button_counter'][1];
  }
}
